@@ -200,13 +200,18 @@ REST_API_VERSION_HISTORY = """
     * 2.80 - Added share backup APIs.
     * 2.81 - Added API methods, endpoint /resource-locks.
     * 2.82 - Added lock and restriction to share access rules.
+    * 2.83 - Added 'disabled_reason' field to services.
+    * 2.84 - Added mount_point_name to shares.
+    * 2.85 - Added backup_type field to share backups.
+    * 2.86 - Add ensure share API.
+    * 2.87 - Added Share export location metadata API
 """
 
 # The minimum and maximum versions of the API supported
 # The default api version request is defined to be the
 # minimum version of the API supported.
 _MIN_API_VERSION = "2.0"
-_MAX_API_VERSION = "2.82"
+_MAX_API_VERSION = "2.87"
 DEFAULT_API_VERSION = _MIN_API_VERSION
 
 
@@ -267,7 +272,7 @@ class APIVersionRequest(utils.ComparableMixin):
 
     @experimental.setter
     def experimental(self, value):
-        if type(value) != bool:
+        if not isinstance(value, bool):
             msg = _('The experimental property must be a bool value.')
             raise exception.InvalidParameterValue(err=msg)
         self._experimental = value
@@ -275,7 +280,7 @@ class APIVersionRequest(utils.ComparableMixin):
     def matches_versioned_method(self, method):
         """Compares this version to that of a versioned method."""
 
-        if type(method) != versioned_method.VersionedMethod:
+        if not isinstance(method, versioned_method.VersionedMethod):
             msg = _('An API version request must be compared '
                     'to a VersionedMethod object.')
             raise exception.InvalidParameterValue(err=msg)
