@@ -45,6 +45,7 @@ maprfs_native_share_opts = [
                default="mapr",
                help='Cluster admin user ssh login name.'),
     cfg.StrOpt('maprfs_ssh_pw',
+               secret=True,
                help='Cluster node SSH login password, '
                     'This parameter is not necessary, if '
                     '\'maprfs_ssh_private_key\' is configured.'),
@@ -206,7 +207,7 @@ class MapRFSNativeShareDriver(driver.ExecuteMixin, driver.ShareDriver):
             raise exception.ShareResourceNotFound(share_id=share['share_id'])
 
     def create_share_from_snapshot(self, context, share, snapshot,
-                                   share_server=None):
+                                   share_server=None, parent_share=None):
         """Creates a share from snapshot."""
         metadata = self.api.get_share_metadata(context,
                                                {'id': share['share_id']})
