@@ -362,3 +362,8 @@ class NetAppCmodeSingleSvmShareDriver(driver.ShareDriver):
 
     def delete_backup(self, context, backup, share, **kwargs):
         return self.library.delete_backup(context, backup, share, **kwargs)
+
+    def update_share_from_metadata(self, context, share, metadata,
+                                   share_server=None):
+        self.library.update_share_from_metadata(
+            context, share, metadata, share_server=share_server)
