@@ -1127,8 +1127,21 @@ class ShareActionsTest(test.TestCase):
             'id': 'fake',
             'access_type': 'ip',
             'access_to': '127.0.0.1',
+            'share_id': 'fake_share_id'
         }
-        self.mock_object(resource_locks.API, 'create')
+        mock_deletion_lock_create = mock.Mock()
+        lock_id = 'fake_lock_id'
+        if lock_deletion:
+            mock_deletion_lock_create = mock.Mock(
+                side_effect=[
+                    {'id': lock_id},
+                    {'id': f'{lock_id}2'},
+                    {'id': f'{lock_id}3'}
+                ]
+            )
+        self.mock_object(
+            resource_locks.API, 'create', mock_deletion_lock_create
+        )
 
         id = 'fake_share_id'
         req = fakes.HTTPRequest.blank(
@@ -1147,6 +1160,10 @@ class ShareActionsTest(test.TestCase):
 
         restrict_calls = []
         if lock_deletion:
+            share_lock_reason = (
+                constants.SHARE_LOCKED_BY_ACCESS_LOCK_REASON %
+                {'lock_id': lock_id}
+            )
             restrict_calls.append(
                 mock.call(
                     context, resource_id=access['id'],
@@ -1156,6 +1173,14 @@ class ShareActionsTest(test.TestCase):
                     lock_reason=lock_reason
                 )
             )
+            restrict_calls.append(
+                mock.call(
+                    context, resource_id=access['share_id'],
+                    resource_type='share',
+                    resource_action=constants.RESOURCE_ACTION_DELETE,
+                    lock_reason=share_lock_reason
+                )
+            )
         if lock_visibility:
             restrict_calls.append(
                 mock.call(
@@ -1245,13 +1270,14 @@ class ShareActionsTest(test.TestCase):
     @ddt.unpack
     def test_allow_access_visibility_restrictions(self, lock_visibility,
                                                   lock_deletion, lock_reason):
-        access = {'id': 'fake'}
+        access = {'id': 'fake', 'share_id': 'fake_share_id'}
+        expected_access = {'access': {'fake_key': 'fake_value'}}
         self.mock_object(share_api.API,
                          'allow_access',
                          mock.Mock(return_value=access))
         self.mock_object(self.controller._access_view_builder, 'view',
-                         mock.Mock(return_value={'access': {'fake': 'fake'}}))
-        self.mock_object(resource_locks.API, 'create')
+                         mock.Mock(return_value=expected_access))
+        self.mock_object(self.controller, '_create_access_locks')
 
         id = 'fake_share_id'
         body = {
@@ -1263,7 +1289,6 @@ class ShareActionsTest(test.TestCase):
                 'lock_reason': lock_reason
             }
         }
-        expected = {'access': {'fake': 'fake'}}
         req = fakes.HTTPRequest.blank(
             '/tenant1/shares/%s/action' % id, version='2.82')
         context = req.environ['manila.context']
@@ -1274,31 +1299,13 @@ class ShareActionsTest(test.TestCase):
             req, id, body, lock_visibility=lock_visibility,
             lock_deletion=lock_deletion, lock_reason=lock_reason)
 
-        self.assertEqual(expected, res)
+        self.assertEqual(expected_access, res)
         self.mock_policy_check.assert_called_once_with(
             context, 'share', 'allow_access')
-        restrict_calls = []
-        if lock_deletion:
-            restrict_calls.append(
-                mock.call(
-                    context, resource_id=access['id'],
-                    resource_type='access_rule',
-                    resource_action=constants.RESOURCE_ACTION_DELETE,
-                    resource=access,
-                    lock_reason=lock_reason
-                )
-            )
-        if lock_visibility:
-            restrict_calls.append(
-                mock.call(
-                    context, resource_id=access['id'],
-                    resource_type='access_rule',
-                    resource_action=constants.RESOURCE_ACTION_SHOW,
-                    resource=access,
-                    lock_reason=lock_reason
-                )
-            )
-        resource_locks.API.create.assert_has_calls(restrict_calls)
+        self.controller._create_access_locks.assert_called_once_with(
+            context, access, lock_deletion=lock_deletion,
+            lock_visibility=lock_visibility, lock_reason=lock_reason
+        )
 
     def test_allow_access_with_network_id(self):
         share_network = db_utils.create_share_network()
@@ -1360,7 +1367,6 @@ class ShareActionsTest(test.TestCase):
 
         self.mock_object(share_api.API, "deny_access", _stub_deny_access)
         self.mock_object(share_api.API, "access_get", _fake_access_get)
-        self.mock_object(self.controller, '_check_for_access_rule_locks')
 
         id = 'fake_share_id'
         body = {"os-deny_access": {"access_id": 'fake_acces_id'}}
@@ -1378,20 +1384,14 @@ class ShareActionsTest(test.TestCase):
         share_network = db_utils.create_share_network()
         share = db_utils.create_share(share_network_id=share_network['id'])
         self.mock_object(share_api.API, 'get', mock.Mock(return_value=share))
-        self.mock_object(self.controller, '_check_for_access_rule_locks')
 
         id = 'fake_share_id'
-        access_data = {"access_id": 'fake_acces_id'}
-        body = {"os-deny_access": access_data}
+        body = {"os-deny_access": {"access_id": 'fake_acces_id'}}
         req = fakes.HTTPRequest.blank('/v1/tenant1/shares/%s/action' % id)
 
         res = self.controller._deny_access(req, id, body)
 
         self.assertEqual(202, res.status_int)
-        self.controller._check_for_access_rule_locks.assert_called_once_with(
-            req.environ['manila.context'], access_data,
-            access_data['access_id'], id
-        )
         self.mock_policy_check.assert_called_once_with(
             req.environ['manila.context'], 'share', 'deny_access')
 
@@ -1401,7 +1401,6 @@ class ShareActionsTest(test.TestCase):
 
         self.mock_object(share_api.API, "deny_access", _stub_deny_access)
         self.mock_object(share_api.API, "access_get", _fake_access_get)
-        self.mock_object(self.controller, '_check_for_access_rule_locks')
 
         id = 'super_fake_share_id'
         body = {"os-deny_access": {"access_id": 'fake_acces_id'}}
@@ -1449,14 +1448,12 @@ class ShareActionsTest(test.TestCase):
         access_id = 'fake_access_id'
         share_id = 'fake_share_id'
 
-        self.mock_object(context, 'elevated', mock.Mock(return_value=context))
         self.controller._check_for_access_rule_locks(
             context, {}, access_id, share_id)
 
         delete_search_opts = {
             'resource_id': access_id,
-            'resource_action': constants.RESOURCE_ACTION_DELETE,
-            'all_projects': True,
+            'resource_action': constants.RESOURCE_ACTION_DELETE
         }
 
         resource_locks.API.get_all.assert_called_once_with(
@@ -1475,7 +1472,6 @@ class ShareActionsTest(test.TestCase):
         access_id = 'fake_access_id'
         share_id = 'fake_share_id'
 
-        self.mock_object(context, 'elevated', mock.Mock(return_value=context))
         self.assertRaises(
             webob.exc.HTTPForbidden,
             self.controller._check_for_access_rule_locks,
@@ -1483,8 +1479,7 @@ class ShareActionsTest(test.TestCase):
 
         delete_search_opts = {
             'resource_id': access_id,
-            'resource_action': constants.RESOURCE_ACTION_DELETE,
-            'all_projects': True,
+            'resource_action': constants.RESOURCE_ACTION_DELETE
         }
 
         resource_locks.API.get_all.assert_called_once_with(
@@ -1509,7 +1504,6 @@ class ShareActionsTest(test.TestCase):
         access_id = 'fake_access_id'
         share_id = 'fake_share_id'
 
-        self.mock_object(context, 'elevated', mock.Mock(return_value=context))
         self.assertRaises(
             webob.exc.HTTPForbidden,
             self.controller._check_for_access_rule_locks,
@@ -1517,8 +1511,7 @@ class ShareActionsTest(test.TestCase):
 
         delete_search_opts = {
             'resource_id': access_id,
-            'resource_action': constants.RESOURCE_ACTION_DELETE,
-            'all_projects': True,
+            'resource_action': constants.RESOURCE_ACTION_DELETE
         }
 
         resource_locks.API.get_all.assert_called_once_with(
@@ -1549,7 +1542,6 @@ class ShareActionsTest(test.TestCase):
         access_id = 'fake_access_id'
         share_id = 'fake_share_id'
 
-        self.mock_object(context, 'elevated', mock.Mock(return_value=context))
         self.assertRaises(
             webob.exc.HTTPForbidden,
             self.controller._check_for_access_rule_locks,
@@ -1557,8 +1549,7 @@ class ShareActionsTest(test.TestCase):
         )
         delete_search_opts = {
             'resource_id': access_id,
-            'resource_action': constants.RESOURCE_ACTION_DELETE,
-            'all_projects': True,
+            'resource_action': constants.RESOURCE_ACTION_DELETE
         }
         resource_locks.API.get_all.assert_called_once_with(
             context, search_opts=delete_search_opts, show_count=True
@@ -1585,19 +1576,15 @@ class ShareActionsTest(test.TestCase):
         access_id = 'fake_access_id'
         share_id = 'fake_share_id'
 
-        self.mock_object(context, 'elevated', mock.Mock(return_value=context))
         self.controller._check_for_access_rule_locks(
             context, {'unrestrict': True}, access_id, share_id)
 
         delete_search_opts = {
             'resource_id': access_id,
-            'resource_action': constants.RESOURCE_ACTION_DELETE,
-            'all_projects': True,
+            'resource_action': constants.RESOURCE_ACTION_DELETE
         }
         resource_locks.API.get_all.assert_called_once_with(
-            context.elevated(), search_opts=delete_search_opts,
-            show_count=True
-        )
+            context, search_opts=delete_search_opts, show_count=True)
         (resource_locks.API.ensure_context_can_delete_lock
             .assert_called_once_with(
                 context, locks[0]['id']))
