@@ -14,8 +14,8 @@
 #    under the License.
 
 import os
-
-import mock
+import time
+from unittest import mock
 
 from manila.data import utils as data_utils
 from manila import exception
@@ -208,6 +208,7 @@ class CopyClassTestCase(test.TestCase):
         self.mock_object(utils, 'execute', mock.Mock(
             side_effect=get_output))
         self.mock_object(self._copy, 'get_progress')
+        self.mock_object(time, 'sleep')
 
         # run
         self._copy.copy_data(self._copy.src)
