@@ -1054,3 +1054,10 @@ class DummyDriver(driver.ShareDriver):
                   {'backup': backup['id'],
                    'share': share_instance['share_id']})
         return {'total_progress': '100'}
+
+    def update_share_from_metadata(self, context, share_instance, metadata,
+                                   share_server=None):
+        LOG.debug("Updated share %(share)s. Metadata %(metadata)s "
+                  "applied successfully.",
+                  {'share': share_instance['share_id'],
+                   'metadata': metadata})
