@@ -30,6 +30,7 @@ from manila import exception
 from manila.share import driver
 from manila.share.drivers.inspur.instorage import cli_helper
 from manila.share.drivers.inspur.instorage import instorage
+from manila import ssh_utils
 from manila import test
 from manila.tests import fake_share
 from manila import utils as manila_utils
@@ -61,6 +62,8 @@ class FakeConfig(object):
             "reserved_share_percentage", 0)
         self.reserved_share_from_snapshot_percentage = kwargs.get(
             "reserved_share_from_snapshot_percentage", 0)
+        self.reserved_share_extend_percentage = kwargs.get(
+            "reserved_share_extend_percentage", 0)
         self.max_over_subscription_ratio = kwargs.get(
             "max_over_subscription_ratio", 0)
         self.filter_function = kwargs.get("filter_function", None)
@@ -140,6 +143,7 @@ class InStorageShareDriverTestCase(test.TestCase):
                 'allocated_capacity_gb': 10,
                 'reserved_percentage': 0,
                 'reserved_snapshot_percentage': 0,
+                'reserved_share_extend_percentage': 0,
                 'qos': False,
                 'dedupe': False,
                 'compression': False,
@@ -164,6 +168,7 @@ class InStorageShareDriverTestCase(test.TestCase):
             'storage_protocol': 'NFS_CIFS',
             'reserved_percentage': 0,
             'reserved_snapshot_percentage': 0,
+            'reserved_share_extend_percentage': 0,
             'max_over_subscription_ratio': 0,
             'snapshot_support': False,
             'create_share_from_snapshot_support': False,
@@ -271,7 +276,7 @@ class SSHRunnerTestCase(test.TestCase):
     def test___call___success(self):
         mock_csi = self.mock_object(manila_utils, 'check_ssh_injection')
         mock_sshpool = mock.Mock(return_value=self.fakePool)
-        self.mock_object(manila_utils, 'SSHPool', mock_sshpool)
+        self.mock_object(ssh_utils, 'SSHPool', mock_sshpool)
         mock_se = mock.Mock(return_value='fake_value')
         self.mock_object(cli_helper.SSHRunner, '_ssh_execute', mock_se)
 
@@ -299,7 +304,7 @@ class SSHRunnerTestCase(test.TestCase):
     def test___call___ssh_pool_failed(self):
         mock_csi = self.mock_object(manila_utils, 'check_ssh_injection')
         mock_sshpool = mock.Mock(side_effect=paramiko.SSHException())
-        self.mock_object(manila_utils, 'SSHPool', mock_sshpool)
+        self.mock_object(ssh_utils, 'SSHPool', mock_sshpool)
 
         runner = cli_helper.SSHRunner(
             '127.0.0.1', '22', 'fakeuser', 'fakepassword'
@@ -311,7 +316,7 @@ class SSHRunnerTestCase(test.TestCase):
     def test___call___ssh_exec_failed(self):
         mock_csi = self.mock_object(manila_utils, 'check_ssh_injection')
         mock_sshpool = mock.Mock(return_value=self.fakePool)
-        self.mock_object(manila_utils, 'SSHPool', mock_sshpool)
+        self.mock_object(ssh_utils, 'SSHPool', mock_sshpool)
         exception = processutils.ProcessExecutionError()
         mock_se = mock.Mock(side_effect=exception)
         self.mock_object(cli_helper.SSHRunner, '_ssh_execute', mock_se)
@@ -910,6 +915,7 @@ class InStorageAssistantTestCase(test.TestCase):
                 'qos': False,
                 'reserved_percentage': 0,
                 'reserved_snapshot_percentage': 0,
+                'reserved_share_extend_percentage': 0,
                 'dedupe': False,
                 'compression': False,
                 'thin_provisioning': False,
