@@ -16,9 +16,9 @@
 import copy
 import errno
 import os
+from unittest import mock
 
 import ddt
-import mock
 from oslo_config import cfg
 
 from manila import context
@@ -92,7 +92,7 @@ class GaneshaNASHelperTestCase(test.TestCase):
                                                 fake_template2]))
         self.mock_object(ganesha.ganesha_utils, 'patch',
                          mock.Mock(side_effect=fake_patch_run))
-        with mock.patch('six.moves.builtins.open',
+        with mock.patch('builtins.open',
                         mock.mock_open()) as mockopen:
             mockopen().read.side_effect = ['fakeconf0', 'fakeconf1']
             ret = self._helper._load_conf_dir(self.fake_conf_dir_path)
@@ -119,7 +119,7 @@ class GaneshaNASHelperTestCase(test.TestCase):
         self.mock_object(ganesha.LOG, 'info')
         self.mock_object(ganesha.ganesha_manager, 'parseconf')
         self.mock_object(ganesha.ganesha_utils, 'patch')
-        with mock.patch('six.moves.builtins.open',
+        with mock.patch('builtins.open',
                         mock.mock_open(read_data='fakeconf')) as mockopen:
             ret = self._helper._load_conf_dir(self.fake_conf_dir_path,
                                               must_exist=False)
@@ -238,15 +238,17 @@ class GaneshaNASHelperTestCase(test.TestCase):
                          mock.Mock(return_value='fakefsal'))
         self.mock_object(ganesha.ganesha_utils, 'patch',
                          mock.Mock(side_effect=fake_patch_run))
+        self.mock_object(ganesha.ganesha_utils, 'validate_access_rule',
+                         mock.Mock(return_value=True))
         ret = self._helper._allow_access(fake_basepath, self.share,
                                          self.access)
         self._helper.ganesha.get_export_id.assert_called_once_with()
         self._helper._fsal_hook.assert_called_once_with(
-            fake_basepath, self.share, self.access)
+            fake_basepath, self.share, self.access, sub_name=None)
         mock_ganesha_utils_patch.assert_called_once_with(
             {}, self._helper.export_template, fake_output_template)
         self._helper._fsal_hook.assert_called_once_with(
-            fake_basepath, self.share, self.access)
+            fake_basepath, self.share, self.access, sub_name=None)
         self._helper.ganesha.add_export.assert_called_once_with(
             fake_export_name, fake_output_template)
         self.assertIsNone(ret)
@@ -308,6 +310,26 @@ class GaneshaNASHelperTestCase(test.TestCase):
         self.assertTrue(self._helper.ganesha.reset_exports.called)
         self.assertTrue(self._helper.ganesha.restart_service.called)
 
+    def test_update_access_invalid_share_access_type(self):
+        bad_rule = fake_share.fake_access(access_type='notip', id='fakeid')
+        expected = {'fakeid': {'state': 'error'}}
+
+        result = self._helper.update_access(self._context, self.share,
+                                            access_rules=[bad_rule],
+                                            add_rules=[], delete_rules=[])
+
+        self.assertEqual(expected, result)
+
+    def test_update_access_invalid_share_access_level(self):
+        bad_rule = fake_share.fake_access(access_level='RO', id='fakeid')
+        expected = {'fakeid': {'state': 'error'}}
+
+        result = self._helper.update_access(self._context, self.share,
+                                            access_rules=[bad_rule],
+                                            add_rules=[], delete_rules=[])
+
+        self.assertEqual(expected, result)
+
 
 @ddt.ddt
 class GaneshaNASHelper2TestCase(test.TestCase):
@@ -322,12 +344,19 @@ class GaneshaNASHelper2TestCase(test.TestCase):
         CONF.set_default('ganesha_export_template_dir',
                          '/fakedir2/faketempl.d')
         CONF.set_default('ganesha_service_name', 'ganesha.fakeservice')
+        CONF.set_default('ganesha_rados_store_enable', True)
+        CONF.set_default('ganesha_rados_store_pool_name', 'ceph_pool')
+        CONF.set_default('ganesha_rados_export_index', 'fake_index')
+        CONF.set_default('ganesha_rados_export_counter', 'fake_counter')
+
         self._context = context.get_admin_context()
         self._execute = mock.Mock(return_value=('', ''))
+        self.rados_client = mock.Mock()
         self.fake_conf = config.Configuration(None)
         self.fake_conf_dir_path = '/fakedir0/exports.d'
         self._helper = ganesha.GaneshaNASHelper2(
-            self._execute, self.fake_conf, tag='faketag')
+            self._execute, self.fake_conf, tag='faketag',
+            rados_client=self.rados_client)
         self._helper.ganesha = mock.Mock()
         self._helper.export_template = {}
         self.share = fake_share.fake_share()
@@ -335,9 +364,100 @@ class GaneshaNASHelper2TestCase(test.TestCase):
         self.rule2 = fake_share.fake_access(access_level='rw',
                                             access_to='10.0.0.2')
 
+    @ddt.data(False, True)
+    def test_init_helper_with_rados_store(self, rados_store_enable):
+        CONF.set_default('ganesha_rados_store_enable', rados_store_enable)
+        mock_template = mock.Mock()
+        mock_ganesha_manager = mock.Mock()
+        self.mock_object(ganesha.ganesha_manager, 'GaneshaManager',
+                         mock.Mock(return_value=mock_ganesha_manager))
+        self.mock_object(self._helper, '_load_conf_dir',
+                         mock.Mock(return_value={}))
+        self.mock_object(self._helper, '_default_config_hook',
+                         mock.Mock(return_value=mock_template))
+
+        ret = self._helper.init_helper()
+
+        if rados_store_enable:
+            kwargs = {
+                'ganesha_config_path': '/fakedir0/fakeconfig',
+                'ganesha_export_dir': '/fakedir0/export.d',
+                'ganesha_service_name': 'ganesha.fakeservice',
+                'ganesha_rados_store_enable': True,
+                'ganesha_rados_store_pool_name': 'ceph_pool',
+                'ganesha_rados_export_index': 'fake_index',
+                'ganesha_rados_export_counter': 'fake_counter',
+                'rados_client': self.rados_client
+            }
+        else:
+            kwargs = {
+                'ganesha_config_path': '/fakedir0/fakeconfig',
+                'ganesha_export_dir': '/fakedir0/export.d',
+                'ganesha_service_name': 'ganesha.fakeservice',
+                'ganesha_db_path': '/fakedir1/fake.db'
+            }
+        ganesha.ganesha_manager.GaneshaManager.assert_called_once_with(
+            self._execute, '<no name>', **kwargs)
+        self._helper._load_conf_dir.assert_called_once_with(
+            '/fakedir2/faketempl.d', must_exist=False)
+        self.assertEqual(mock_ganesha_manager, self._helper.ganesha)
+        self._helper._default_config_hook.assert_called_once_with()
+        self.assertEqual(mock_template, self._helper.export_template)
+        self.assertIsNone(ret)
+
+    @ddt.data(False, True)
+    def test_init_helper_conf_dir_empty(self, conf_dir_empty):
+        mock_template = mock.Mock()
+        mock_ganesha_manager = mock.Mock()
+        self.mock_object(ganesha.ganesha_manager, 'GaneshaManager',
+                         mock.Mock(return_value=mock_ganesha_manager))
+        if conf_dir_empty:
+            self.mock_object(self._helper, '_load_conf_dir',
+                             mock.Mock(return_value={}))
+        else:
+            self.mock_object(self._helper, '_load_conf_dir',
+                             mock.Mock(return_value=mock_template))
+        self.mock_object(self._helper, '_default_config_hook',
+                         mock.Mock(return_value=mock_template))
+
+        ret = self._helper.init_helper()
+
+        ganesha.ganesha_manager.GaneshaManager.assert_called_once_with(
+            self._execute, '<no name>',
+            ganesha_config_path='/fakedir0/fakeconfig',
+            ganesha_export_dir='/fakedir0/export.d',
+            ganesha_service_name='ganesha.fakeservice',
+            ganesha_rados_store_enable=True,
+            ganesha_rados_store_pool_name='ceph_pool',
+            ganesha_rados_export_index='fake_index',
+            ganesha_rados_export_counter='fake_counter',
+            rados_client=self.rados_client)
+        self._helper._load_conf_dir.assert_called_once_with(
+            '/fakedir2/faketempl.d', must_exist=False)
+        self.assertEqual(mock_ganesha_manager, self._helper.ganesha)
+        if conf_dir_empty:
+            self._helper._default_config_hook.assert_called_once_with()
+        else:
+            self.assertFalse(self._helper._default_config_hook.called)
+        self.assertEqual(mock_template, self._helper.export_template)
+        self.assertIsNone(ret)
+
+    def test_init_helper_with_rados_store_pool_name_not_set(self):
+        self.mock_object(ganesha.ganesha_manager, 'GaneshaManager')
+        self.mock_object(self._helper, '_load_conf_dir')
+        self.mock_object(self._helper, '_default_config_hook')
+        self._helper.configuration.ganesha_rados_store_pool_name = None
+
+        self.assertRaises(
+            exception.GaneshaException, self._helper.init_helper)
+
+        self.assertFalse(ganesha.ganesha_manager.GaneshaManager.called)
+        self.assertFalse(self._helper._load_conf_dir.called)
+        self.assertFalse(self._helper._default_config_hook.called)
+
     def test_update_access_add_export(self):
         mock_gh = self._helper.ganesha
-        self.mock_object(mock_gh, '_check_export_file_exists',
+        self.mock_object(mock_gh, 'check_export_exists',
                          mock.Mock(return_value=False))
         self.mock_object(mock_gh, 'get_export_id',
                          mock.Mock(return_value=100))
@@ -347,6 +467,8 @@ class GaneshaNASHelper2TestCase(test.TestCase):
                          mock.Mock(return_value='/fakepath'))
         self.mock_object(self._helper, '_fsal_hook',
                          mock.Mock(return_value={'Name': 'fake'}))
+        self.mock_object(ganesha.ganesha_utils, 'validate_access_rule',
+                         mock.Mock(return_value=True))
         result_confdict = {
             'EXPORT': {
                 'Export_Id': 100,
@@ -364,13 +486,14 @@ class GaneshaNASHelper2TestCase(test.TestCase):
             self._context, self.share, access_rules=[self.rule1],
             add_rules=[], delete_rules=[])
 
-        mock_gh._check_export_file_exists.assert_called_once_with('fakename')
+        mock_gh.check_export_exists.assert_called_once_with('fakename')
         mock_gh.get_export_id.assert_called_once_with()
-        self._helper._get_export_path.assert_called_once_with(self.share)
+        self._helper._get_export_path.assert_called_once_with(
+            self.share, sub_name=None)
         (self._helper._get_export_pseudo_path.assert_called_once_with(
-            self.share))
+            self.share, sub_name=None))
         self._helper._fsal_hook.assert_called_once_with(
-            None, self.share, None)
+            None, self.share, None, sub_name=None)
         mock_gh.add_export.assert_called_once_with(
             'fakename', result_confdict)
         self.assertFalse(mock_gh.update_export.called)
@@ -380,12 +503,14 @@ class GaneshaNASHelper2TestCase(test.TestCase):
               [{'Access_Type': 'ro', 'Clients': '10.0.0.1'}])
     def test_update_access_update_export(self, client):
         mock_gh = self._helper.ganesha
-        self.mock_object(mock_gh, '_check_export_file_exists',
+        self.mock_object(mock_gh, 'check_export_exists',
                          mock.Mock(return_value=True))
         self.mock_object(
-            mock_gh, '_read_export_file',
+            mock_gh, '_read_export',
             mock.Mock(return_value={'EXPORT': {'CLIENT': client}})
         )
+        self.mock_object(ganesha.ganesha_utils, 'validate_access_rule',
+                         mock.Mock(return_value=True))
         result_confdict = {
             'EXPORT': {
                 'CLIENT': [
@@ -398,7 +523,7 @@ class GaneshaNASHelper2TestCase(test.TestCase):
             self._context, self.share, access_rules=[self.rule1, self.rule2],
             add_rules=[self.rule2], delete_rules=[])
 
-        mock_gh._check_export_file_exists.assert_called_once_with('fakename')
+        mock_gh.check_export_exists.assert_called_once_with('fakename')
         mock_gh.update_export.assert_called_once_with('fakename',
                                                       result_confdict)
         self.assertFalse(mock_gh.add_export.called)
@@ -406,12 +531,12 @@ class GaneshaNASHelper2TestCase(test.TestCase):
 
     def test_update_access_remove_export(self):
         mock_gh = self._helper.ganesha
-        self.mock_object(mock_gh, '_check_export_file_exists',
+        self.mock_object(mock_gh, 'check_export_exists',
                          mock.Mock(return_value=True))
         self.mock_object(self._helper, '_cleanup_fsal_hook')
         client = {'Access_Type': 'ro', 'Clients': '10.0.0.1'}
         self.mock_object(
-            mock_gh, '_read_export_file',
+            mock_gh, '_read_export',
             mock.Mock(return_value={'EXPORT': {'CLIENT': client}})
         )
 
@@ -419,16 +544,16 @@ class GaneshaNASHelper2TestCase(test.TestCase):
             self._context, self.share, access_rules=[],
             add_rules=[], delete_rules=[self.rule1])
 
-        mock_gh._check_export_file_exists.assert_called_once_with('fakename')
+        mock_gh.check_export_exists.assert_called_once_with('fakename')
         mock_gh.remove_export.assert_called_once_with('fakename')
         self._helper._cleanup_fsal_hook.assert_called_once_with(
-            None, self.share, None)
+            None, self.share, None, sub_name=None)
         self.assertFalse(mock_gh.add_export.called)
         self.assertFalse(mock_gh.update_export.called)
 
     def test_update_access_export_file_already_removed(self):
         mock_gh = self._helper.ganesha
-        self.mock_object(mock_gh, '_check_export_file_exists',
+        self.mock_object(mock_gh, 'check_export_exists',
                          mock.Mock(return_value=False))
         self.mock_object(ganesha.LOG, 'warning')
         self.mock_object(self._helper, '_cleanup_fsal_hook')
@@ -437,9 +562,36 @@ class GaneshaNASHelper2TestCase(test.TestCase):
             self._context, self.share, access_rules=[],
             add_rules=[], delete_rules=[self.rule1])
 
-        mock_gh._check_export_file_exists.assert_called_once_with('fakename')
+        mock_gh.check_export_exists.assert_called_once_with('fakename')
         ganesha.LOG.warning.assert_called_once_with(mock.ANY, mock.ANY)
         self.assertFalse(mock_gh.add_export.called)
         self.assertFalse(mock_gh.update_export.called)
         self.assertFalse(mock_gh.remove_export.called)
         self.assertFalse(self._helper._cleanup_fsal_hook.called)
+
+    def test_update_access_invalid_share_access_type(self):
+        mock_gh = self._helper.ganesha
+        self.mock_object(mock_gh, 'check_export_exists',
+                         mock.Mock(return_value=False))
+        bad_rule = fake_share.fake_access(access_type='notip', id='fakeid')
+        expected = {'fakeid': {'state': 'error'}}
+
+        result = self._helper.update_access(self._context, self.share,
+                                            access_rules=[bad_rule],
+                                            add_rules=[], delete_rules=[])
+
+        self.assertEqual(expected, result)
+
+    def test_update_access_invalid_share_access_level(self):
+        bad_rule = fake_share.fake_access(access_level='NOT_RO_OR_RW',
+                                          id='fakeid')
+        expected = {'fakeid': {'state': 'error'}}
+        mock_gh = self._helper.ganesha
+        self.mock_object(mock_gh, 'check_export_exists',
+                         mock.Mock(return_value=False))
+
+        result = self._helper.update_access(self._context, self.share,
+                                            access_rules=[bad_rule],
+                                            add_rules=[], delete_rules=[])
+
+        self.assertEqual(expected, result)
